@@ -10,13 +10,12 @@ use MIME::Base64 qw(encode_base64);
 use parent qw(Plack::Middleware::StackTrace);
 use Try::Tiny;
 
-our $VERSION = "0.02";
+our $VERSION = "0.03";
 
 sub call {
     my($self, $env) = @_;
 
-    my $last_key = '';
-    my %seen;
+    my ($last_key, %seen);
     local $SIG{__DIE__} = sub {
         my $key = _make_key($_[0]);
         my $list = $seen{$key} || [];
@@ -27,6 +26,7 @@ sub call {
             indent => 1,
             message => munge_error($_[0], [ caller ]),
             ignore_package => __PACKAGE__,
+            no_refs => 1,
         );
         $seen{$key} = $list;
         $last_key = $key;
@@ -42,8 +42,9 @@ sub call {
         _error('text/plain', $caught, 'no_trace');
     };
 
-    my $trace = $seen{$last_key} || [];
-    if (scalar @$trace && $self->should_show_trace($caught, $last_key, $res)) {
+    my $trace = $self->force ? $seen{$last_key} : $seen{_make_key($caught)};
+    $trace ||= [];
+    if (scalar @$trace && $self->should_show_trace($caught, $res)) {
         my $text = $trace->[0]->as_string;
         my $html = @$trace > 1
             ? _multi_trace_html(@$trace) : $trace->[0]->as_html;
@@ -58,18 +59,14 @@ sub call {
     # $trace has refs to Standalone.pm's args ($conn etc.) and
     # prevents garbage collection to be happening.
     undef %seen;
-    undef $last_key;
 
     return $res;
 }
 
 sub should_show_trace {
-    my ($self, $err, $key, $res) = @_;
-    if ($err) {
-        return _make_key($err) eq $key;
-    } else {
-        return $self->force && ref $res eq 'ARRAY' && $res->[0] == 500;
-    }
+    my ($self, $err, $res) = @_;
+    return 1 if $err;
+    return $self->force && ref $res eq 'ARRAY' && $res->[0] == 500;
 }
 
 sub no_trace_error { Plack::Middleware::StackTrace::no_trace_error(@_) }
@@ -206,22 +203,47 @@ Plack::Middleware::StackTrace::RethrowFriendly - Display the original stack trac
 =head1 DESCRIPTION
 
 This middleware is the same as L<Plack::Middleware::StackTrace> except
-that if you catch (C<eval> or C<try>-C<catch> for example) an error
-and rethrow (C<die> or C<croak> for example) it, the original stack
-trace not the rethrown one is displayed.
+that additional information for rethrown errors are available for HTML
+stack trace.
 
-When the response is displayed as an HTML, all the errors including
+If you catch (C<eval> or C<try>-C<catch> for example) an error and
+rethrow (C<die> or C<croak> for example) it, all the errors including
 rethrown ones are visible through the throwing point selector at the
 top of the HTML.
 
+For example, consider the following code.
+
+  sub fail {
+      die 'foo';
+  }
+
+  sub another {
+      fail();
+  }
+
+  builder {
+      enable 'StackTrace';
+
+      sub {
+          eval { fail() }; # (1)
+          another();       # (2)
+
+          return [ 200, [ 'Content-Type' => 'text/plain' ], [ 'OK' ] ];
+      };
+  };
+
+L<Plack::Middleware::StackTrace> blames (1) since it is the first
+place where C<'foo'> is raised.  This behavior may be misleading if
+the real culprit was something done in C<another>.
+
+C<Plack::Middleware::StackTrace::RethrowFriendly> displays stack
+traces of both (1) and (2) in each page and (1) is selected by
+default.
+
 =head1 SEE ALSO
 
 L<Plack::Middleware::StackTrace>
 
-=head1 ACKNOWLEDGMENT
-
-This implementation is a fork from L<a patch to Plack::Middleware::StackTrace|https://github.com/plack/Plack/compare/original-stacktrace> by Jesse Luehrs.
-
 =head1 LICENSE
 
 Copyright (C) TOYAMA Nao and INA Lintaro
